/* FGGE - Fabiogiopla Game Engine
 * Licensed under MIT license.
 * http://fabiogiopla.altervista.org/?cat=software/dev_utils/fgge
 * Copyright (c) Fabio Iotti 2012. Some rights reserved.
 */

var GameEngine = {
	// Static Classes
	Renderer: {
		// Classes
		Model:function(path) {
			// Attributes
			this.x = 0;
			this.y = 0;
			this.transparency = 1;
			this.rotation = 0;
			this.scaleX = 1;
			this.scaleY = 1;
			
			// MODEL IMAGE START
			var imageModel = new Image();
			imageModel.src = path;
			GameEngine._totalObjects++;
			imageModel.onload = function() {
				GameEngine._loadedObjects++;
				GameEngine._resetGameFunction();
			};
			imageModel.onerror = function() {
				alert("WARNING: Failed to load \""+imageModel.src+"\" image!");
				
				GameEngine._loadedObjects++;
				GameEngine._resetGameFunction();
			};
			// MODEL IMAGE END
			
			// Methods
			this.render = function() {
				GameEngine.Renderer.context2D.globalAlpha = this.transparency;
				
				if(this.rotation || this.scaleX != 1 || this.scaleY != 1) {
					GameEngine.Renderer.context2D.save();
					
					GameEngine.Renderer.context2D.translate(
						this.x,
						this.y
					);
					
					GameEngine.Renderer.context2D.rotate(this.rotation);
					
					GameEngine.Renderer.context2D.drawImage(
						imageModel,
						0,
						0,
						imageModel.width,
						imageModel.height,
						(-imageModel.width*this.scaleX)/2,
						(-imageModel.height*this.scaleY)/2,
						imageModel.width*this.scaleX,
						imageModel.height*this.scaleY
					);
					
					GameEngine.Renderer.context2D.restore();
				} else {
					GameEngine.Renderer.context2D.drawImage(
						imageModel,
						this.x - imageModel.width/2,
						this.y - imageModel.height/2
					);
				}
			}
		},
		
		// Attributes
		//canvas
		//context2D
		
		// Methods
		generateCanvas:function() {
			GameEngine.Renderer.canvas = document.createElement("canvas");
			GameEngine.Renderer.context2D = GameEngine.Renderer.canvas.getContext('2d');
			document.getElementsByTagName("script")[document.getElementsByTagName("script").length-1].parentNode.appendChild(GameEngine.Renderer.canvas);
		
			GameEngine.Renderer.canvas.oncontextmenu = function() {return false;};
		},
		
		clearScreen:function() {
			GameEngine.Renderer.context2D.clearRect(0, 0, GameEngine.Renderer.canvas.width, GameEngine.Renderer.canvas.height);
		},
		
		setBackgroundColor:function(color) {
			GameEngine.Renderer.canvas.style.backgroundColor = color;
		},
		
		setCanvasSize:function(width, height) {
			GameEngine.Renderer.canvas.style.width = width;
			GameEngine.Renderer.canvas.style.height = height;
			
			GameEngine.Renderer.canvas.width = width;
			GameEngine.Renderer.canvas.height = height;
		},
		
		setFramerate:function(fps) {
			GameEngine._framerate = fps;
			GameEngine._resetGameFunction();
		}
	},
	
	Keyboard: {
		// Static Classes
		KeyboardEvent: {},
		
		// Classes
		KeyboardEvent:function(keyNumber, eventType) {
			var keyNum = keyNumber;
			var evType = eventType;
			
			// Methods
			this.getKeyNumber = function() {
				return keyNum;
			},
			
			this.getEventType = function() {
				return evType;
			}
		},
		
		// Attributes
		//_keyboardListeners
		
		// Methods
		addKeyboardListener:function(listener) {
			GameEngine.Keyboard._keyboardListeners.push(listener);
		},
		
		removeKeyboardListener:function(listener) {
			GameEngine._removeArrayElement(GameEngine.Keyboard._keyboardListeners, listener);
		},
		
		_initialize:function() {
			GameEngine.Keyboard.KeyboardEvent.EVENT_PRESS = 0;
			GameEngine.Keyboard.KeyboardEvent.EVENT_RELEASE = 1;
			GameEngine.Keyboard._keyboardListeners = new Array();

			function upDateKeyDown(e) {
				GameEngine._sendEventToElements(GameEngine.Keyboard._keyboardListeners, new GameEngine.Keyboard.KeyboardEvent(e.keyCode, GameEngine.Keyboard.KeyboardEvent.EVENT_PRESS));
			}
			function upDateKeyUp(e) {
				GameEngine._sendEventToElements(GameEngine.Keyboard._keyboardListeners, new GameEngine.Keyboard.KeyboardEvent(e.keyCode, GameEngine.Keyboard.KeyboardEvent.EVENT_RELEASE));
			}
			
			document.addEventListener("keydown", upDateKeyDown, false);
			document.addEventListener("keyup", upDateKeyUp, false);
		}
	},
	
	Mouse: {
		// Static Classes
		MouseEvent: {},
		
		// Classes
		MouseEvent:function(buttonNumber, eventType) {
			var btnNum = buttonNumber;
			var evType = eventType;
			
			// Methods
			this.getButtonNumber = function() {
				return btnNum;
			},
			
			this.getEventType = function() {
				return evType;
			}
		},
		
		// Attibutes
		//x
		//y
		//_mouseListeners
		
		// Methods
		addMouseListener:function(listener) {
			GameEngine.Mouse._mouseListeners.push(listener);
		},
		
		removeMouseListener:function(listener) {
			GameEngine._removeArrayElement(GameEngine.Mouse._mouseListeners, listener);
		},
		
		_initialize:function() {
			GameEngine.Mouse.MouseEvent.EVENT_PRESS = 0;
			GameEngine.Mouse.MouseEvent.EVENT_RELEASE = 1;
			GameEngine.Mouse.MouseEvent.EVENT_MOVE = 2;
			GameEngine.Mouse.MouseEvent.EVENT_WHEEL = 3;
			GameEngine.Mouse.MouseEvent.MOUSE_LEFT = 0;
			GameEngine.Mouse.MouseEvent.MOUSE_RIGHT = 2;
			GameEngine.Mouse.MouseEvent.MOUSE_MIDDLE = 1;
			GameEngine.Mouse._mouseListeners = new Array();
		},
		
		_gamestartInitializeOnce:function() {
			function upDateMouse(e) {
				var pos = GameEngine._getRelativePosition(e);
				GameEngine.Mouse.x = pos[0];
				GameEngine.Mouse.y = pos[1];
				
				GameEngine._sendEventToElements(GameEngine.Mouse._mouseListeners, new GameEngine.Mouse.MouseEvent(-1, GameEngine.Mouse.MouseEvent.EVENT_MOVE));
			}
			function upDateMouseDown(e) {
				GameEngine._sendEventToElements(GameEngine.Mouse._mouseListeners, new GameEngine.Mouse.MouseEvent(e.button, GameEngine.Mouse.MouseEvent.EVENT_PRESS));
			}
			function upDateMouseUp(e) {
				GameEngine._sendEventToElements(GameEngine.Mouse._mouseListeners, new GameEngine.Mouse.MouseEvent(e.button, GameEngine.Mouse.MouseEvent.EVENT_RELEASE));
			}
			function upDateWheel(e) {
				var evt = window.event || e;
				var delta = (evt.detail)?(evt.detail*(-120)):(evt.wheelDelta);
				
				GameEngine._sendEventToElements(GameEngine.Mouse._mouseListeners, new GameEngine.Mouse.MouseEvent(delta/120, GameEngine.Mouse.MouseEvent.EVENT_WHEEL));
				return false;
			}
			
			// SINPPET: http://www.javascriptkit.com/javatutors/onmousewheel.shtml
			var mousewheelevt = (/Firefox/i.test(navigator.userAgent))?("DOMMouseScroll"):("mousewheel");
			
			if(document.attachEvent)
				document.attachEvent("on"+mousewheelevt, upDateWheel);
			else if(document.addEventListener)
				document.addEventListener(mousewheelevt, upDateWheel, false);
			
			GameEngine.Renderer.canvas.addEventListener("mousemove", upDateMouse, false);
			GameEngine.Renderer.canvas.addEventListener("mouseup", upDateMouseUp, false);
			GameEngine.Renderer.canvas.addEventListener("mousedown", upDateMouseDown, false);
		}
	},
	
	Saving: {
		// Methods
		save:function(name, value) {
			localStorage[name] = value;
		},
		
		load:function(name) {
			return localStorage[name];
		}
	},
	
	// Attributes
	//_totalObjects
	//_loadedObjects
	//_gameFunction
	//_gameCycleIntervalID
	//_framerate
	//_firstTry
	
	// Methods
	startGame:function(gameFunction) {
		GameEngine._gameFunction = gameFunction;
		GameEngine._resetGameFunction();
	},
	
	_resetGameFunction:function() {
		if(GameEngine._gameCycleIntervalID) {
			clearInterval(GameEngine._gameCycleIntervalID);
			GameEngine._gameCycleIntervalID = undefined;
		}
		
		if(GameEngine._loadedObjects >= GameEngine._totalObjects && GameEngine._gameFunction) {
			if(GameEngine._firstTry) {
				GameEngine.Mouse._gamestartInitializeOnce();
				GameEngine._firstTry = false;
			}
			
			GameEngine._gameCycleIntervalID = setInterval(function() {
				try {
					GameEngine._gameFunction();
				} catch(error) {
					clearInterval(GameEngine._gameCycleIntervalID);
					GameEngine._gameCycleIntervalID = undefined;
					
					alert(error);
				}
			}, 1000/GameEngine._framerate);
		}
	},
	
	_initialize:function() {
		GameEngine._totalObjects = 0;
		GameEngine._loadedObjects = 0;
		GameEngine._framerate = 20;
		GameEngine._firstTry = true;
		
		GameEngine.Keyboard._initialize();
		GameEngine.Mouse._initialize();
	},
	
	_removeArrayElement:function(arrayName, arrayElement) {
		for(var i=0; i<arrayName.length; i++) { 
			if(arrayName[i]==arrayElement)
				arrayName.splice(i,1);
		} 
	},
	
	_sendEventToElements:function(array, event) {
		for (var i = 0; i < array.length; i++) {
			array[i](event);
		}
	},
	
	_getRelativePosition:function(e) {
		// SNIPPET: http://tinyurl.com/3r6kcqf
		var x, y;
		if (e.pageX || e.pageY) {
			x = e.pageX;
			y = e.pageY;
		} else {
			x = e.clientX + document.body.scrollLeft + document.documentElement.scrollLeft;
			y = e.clientY + document.body.scrollTop + document.documentElement.scrollTop;
		}

		x -= GameEngine.Renderer.canvas.offsetLeft;
		y -= GameEngine.Renderer.canvas.offsetTop;

		return [x,y];
	}
}
GameEngine._initialize();